﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


namespace Kupo
{
    [AttributeUsage(AttributeTargets.Method, Inherited = false, AllowMultiple = true)]
    internal sealed class IgnoreBehaviorCountAttribute : Attribute
    {
        public IgnoreBehaviorCountAttribute(BehaviorType type)
        {
            Type = type;
        }

        public BehaviorType Type { get; private set; }
    }

    [AttributeUsage(AttributeTargets.Method, Inherited = false, AllowMultiple = true)]
    internal sealed class BehaviorAttribute : Attribute
    {
        public BehaviorAttribute(BehaviorType type, GameContext context = GameContext.All, int priority = -1)
        {
            Type = type;
            SpecificContext = context;


            if (priority < 0)
            {
                PriorityLevel = -flagCount;
            }
            else
                PriorityLevel = priority;
        }


        private int flagCount
        {
            get
            {


                int v = (int)SpecificContext;
                v = v - ((v >> 1) & 0x55555555); // reuse input as temporary
                v = (v & 0x33333333) + ((v >> 2) & 0x33333333); // temp
                int c = ((v + (v >> 4) & 0xF0F0F0F) * 0x1010101) >> 24; // count
                return c;
            }
        }

        public BehaviorType Type { get; private set; }
        public GameContext SpecificContext { get; private set; }
        public int PriorityLevel { get; private set; }
    }
}