﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;
using ff14bot;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.Objects;
using Kupo.Helpers;
using TreeSharp;
using Action = TreeSharp.Action;

namespace Kupo.Rotations
{
    public class PugilistMonk : KupoRoutine
    {
        //DEVELOPERS REPLACE GetType().Name WITH YOUR CR'S NAME.
        public override string Name
        {
            get { return "Kupo [" + GetType().Name + "]"; }
        }

        public override float PullRange
        {
            get { return 2.5f; }
        }

        public override ClassJobType[] Class
        {
            get { return new ClassJobType[] {ClassJobType.Pugilist, ClassJobType.Monk,}; }
        }

        [Behavior(BehaviorType.CombatBuffs)]
        public Composite CreateBasicCombatBuffs()
        {
            return new PrioritySelector(SummonChocobo());
        }
        [Behavior(BehaviorType.PreCombatBuffs)]
        public Composite CreateBasicPreCombatBuffs()
        {
            return SummonChocobo();
        }

        [Behavior(BehaviorType.Heal)]
        public Composite CreateBasicHeal()
        {
            return new PrioritySelector(
                Spell.Cast("Bloodbath", r => Core.Player.CurrentHealthPercent < 60, r => Core.Player),
                Spell.Cast("Second Wind", r => Core.Player.CurrentHealthPercent < 40,r=>Core.Player)
                );
        }

        [Behavior(BehaviorType.Rest)]
        public Composite CreateBasicRest()
        {
            return DefaultRestBehavior(r => Core.Player.CurrentTPPercent);
        }

        [Behavior(BehaviorType.Pull)]
        public Composite CreateBasicPull()
        {
                        return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(ctx => ctx != null,new PrioritySelector(
                    new Decorator(ctx => !RoutineManager.IsAnyDisallowed(CapabilityFlags.Movement), new PrioritySelector(
                CommonBehaviors.MoveToLos(ctx => ctx as GameObject),
                CommonBehaviors.MoveAndStop(ctx => (ctx as GameObject).Location, ctx => Core.Player.CombatReach + PullRange + (ctx as GameObject).CombatReach, true, "Moving to unit")
                    )),


                    Spell.PullCast(r=>"Twin Snakes", r => !Core.Player.HasAura("Disciplined Fist", true, 3600)),
                    Spell.PullCast(r=>"Dragon Kick", r => currentForm == ActiveForm.OpoOpo && !Core.Player.HasAura("Leaden Fist")),
                    Spell.PullCast(r=>"Snap Punch"),
                    Spell.PullCast(r=>"True Strike"),
                    Spell.PullCast(r=>"Bootshine", r => true)
                )));
        }





        private enum ActiveForm
        {
            None,
            Raptor,
            Coeurl,
            OpoOpo
        }
        private static FrameCachedValue<ActiveForm> currentForm = new FrameCachedValue<ActiveForm>(() =>
        {

            var player = Core.Player;

            if (player.HasAura("Raptor Form"))
                return ActiveForm.Raptor;
            if (player.HasAura("Coeurl Form"))
                return ActiveForm.Coeurl;
            if (player.HasAura("Opo-opo Form"))
                return ActiveForm.OpoOpo;

            return ActiveForm.None;
        });


        private static FrameCachedValue<uint> perfectBalance = new FrameCachedValue<uint>(() =>
        {

            var aura = Core.Player.GetAuraByName("Perfect Balance");

            if (aura != null)
                return aura.Value;


            return 0;
        });

        //private static uint perfectBalance => _perfectBalance;




        private Composite SolarRotation
        {
            get
            {
                return new PrioritySelector(
                    Spell.Cast("Twin Snakes", r => perfectBalance == 3),
                    Spell.Cast("Dragon Kick", r => perfectBalance == 2),
                    Spell.Cast("Demolish", r => perfectBalance == 1)
                );
            }
        }

        private Composite LunarRotation
        {
            get
            {

                return new PrioritySelector(
                    Spell.Cast("Dragon Kick", r => perfectBalance == 3),
                    Spell.Cast("Bootshine", r => perfectBalance == 2),
                    Spell.Cast("Dragon Kick", r => perfectBalance == 1)
                );
            }
        }

        private Composite BlitzRotation
        {
            get
            {

                return new PrioritySelector(
                    Spell.Cast("Twin Snakes", r => perfectBalance == 3),
                    Spell.Cast("Dragon Kick", r => perfectBalance == 2),
                    Spell.Cast("Demolish", r => perfectBalance == 1)
                );
            }
        }

        [Behavior(BehaviorType.Combat)]
        public Composite CreateBasicCombat()
        {
            return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(ctx => ctx != null,
                    new PrioritySelector(
                        new Decorator(ctx => !RoutineManager.IsAnyDisallowed(CapabilityFlags.Movement), new PrioritySelector(
                        CommonBehaviors.MoveToLos(ctx => ctx as GameObject),
                        CommonBehaviors.MoveAndStop(ctx => (ctx as GameObject).Location, ctx => Core.Player.CombatReach + PullRange + (ctx as GameObject).CombatReach, true, "Moving to unit")
                        )),


                        Spell.Cast("Steel Peak", r => ActionResourceManager.Monk.ChakraCount == 5),
                        Spell.Cast("Masterful Blitz"),

                        Spell.Cast("Perfect Balance", r=> perfectBalance == 0),


                        new Decorator(r=> perfectBalance > 0, new PrioritySelector(
                            new Decorator(r=> !ActionResourceManager.Monk.ActiveNadi.HasFlag(ActionResourceManager.Monk.Nadi.Solar), SolarRotation),
                            new Decorator(r => !ActionResourceManager.Monk.ActiveNadi.HasFlag(ActionResourceManager.Monk.Nadi.Lunar), LunarRotation),
                            new Decorator(r => ActionResourceManager.Monk.ActiveNadi.HasFlag(ActionResourceManager.Monk.Nadi.Both), BlitzRotation)

                            )),


                        Spell.Cast("Twin Snakes", r => !Core.Player.HasAura("Disciplined Fist", true, 3600)),
                        Spell.Apply("Demolish"),


                        Spell.Cast("Dragon Kick", r => currentForm == ActiveForm.OpoOpo && !Core.Player.HasAura("Leaden Fist")),
                        
                        Spell.Cast("Snap Punch"),
                        Spell.Cast("True Strike"),
                        Spell.Cast("Bootshine", r => true)
                        )));
        }
    }
}