﻿using System;
using System.CodeDom;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms.VisualStyles;
using ff14bot;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.Objects;
using Kupo;
using Kupo.Helpers;
using Kupo.Settings;
using Pathfinding;
using TreeSharp;
using AvailablePetFlags = ff14bot.Managers.ActionResourceManager.Summoner.AvailablePetFlags;
using SummonerARM = ff14bot.Managers.ActionResourceManager.Summoner;
using Action = TreeSharp.Action;

namespace Kupo.Rotations
{
    internal class Summoner : KupoRoutine
    {
        //DEVELOPERS REPLACE GetType().Name WITH YOUR CR'S NAME.
        public override string Name
        {
            get { return "Kupo [" + GetType().Name + "]"; }
        }

        public override void OnInitialize()
        {
            WindowSettings = new SummonerSettings();
        }


        private SummonerSettings settings
        {
            get { return WindowSettings as SummonerSettings; }
        }






        public override float PullRange
        {
            get { return 20; }
        }

        public override ClassJobType[] Class
        {
            get { return new[] { ClassJobType.Summoner }; }
        }


        [Behavior(BehaviorType.CombatBuffs)]
        public Composite CreateBasicCombatBuffs()
        {
            return new PrioritySelector(SummonChocobo());
        }
        [Behavior(BehaviorType.PreCombatBuffs)]
        public Composite CreateBasicPreCombatBuffs()
        {
            return SummonChocobo();
        }

        [Behavior(BehaviorType.Rest)]
        public Composite CreateBasicRest()
        {
            return new PrioritySelector(
                new Decorator(r => Core.Me.SpellCastInfo != null && Core.Me.SpellCastInfo.Name == "Physick" && !Resting, new Action(r => ActionManager.StopCasting())),
                DefaultRestBehavior(r => Core.Player.CurrentManaPercent));
        }

        [Behavior(BehaviorType.Heal)]
        public Composite CreateBasicHeal()
        {
            return new PrioritySelector(
                Spell.Cast("Physick", r => Core.Player.CurrentHealthPercent <= 40, r => Core.Player),
                Spell.Cast("Radiant Aegis", r => Core.Player.CurrentHealthPercent <= 80, r => Core.Player)

                );
        }

        [Behavior(BehaviorType.Pull)]
        public Composite CreateBasicPull()
        {
            return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(ctx => ctx != null, new PrioritySelector(


                EnsureTarget,
                //Stop double casting due to delay
                new Decorator(r => Core.Me.SpellCastInfo != null && Core.Me.SpellCastInfo.SpellData.Name.Equals("Summon Carbuncle") && Core.Player.Pet != null, new Action(r => ActionManager.StopCasting())),
                Spell.Cast(r => "Summon Carbuncle", r => !hasPet && ActionManager.HasSpell("Summon Carbuncle"), r => Core.Player),

                new Decorator(ctx => !RoutineManager.IsAnyDisallowed(CapabilityFlags.Movement), new PrioritySelector(
                    CommonBehaviors.MoveToLos(ctx => ctx as GameObject),
                    CommonBehaviors.MoveAndStop(ctx => (ctx as GameObject).Location, PullRange, true, "Moving to unit")
                )),

                Spell.PullCast("Gemshine"),
                Spell.PullCast("Ruin")
                )));
        }






        private FrameCachedObject<string> WhichPet = new FrameCachedObject<string>(() => {

            if (SummonerARM.ActivePet != SummonerARM.ActivePetType.None)
                return "";

            var value = SummonerARM.AvailablePets;
            if (value.HasFlag(AvailablePetFlags.Ifrit))
                return "Summon Ruby";
            if (value.HasFlag(AvailablePetFlags.Titan))
                return "Summon Topaz";
            if (value.HasFlag(AvailablePetFlags.Garuda))
                return "Summon Emerald";

            return "";
        });


        public static bool hasPet => GameObjectManager.PetObjectId != GameObjectManager.EmptyGameObject;

        [Behavior(BehaviorType.Combat)]
        public Composite CreateBasicCombat()
        {
            return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(ctx => ctx != null,
                    new PrioritySelector(
                         new Decorator(ctx => !RoutineManager.IsAnyDisallowed(CapabilityFlags.Movement), CommonBehaviors.MoveToLos(ctx => ctx as GameObject)),
                        new Decorator(r => !Core.Player.IsCasting, CommonBehaviors.MoveAndStop(ctx => (ctx as GameObject).Location, PullRange, true, "Moving to unit")),

                         new Decorator(r => Core.Me.SpellCastInfo != null && Core.Me.SpellCastInfo.SpellData.Name.Equals("Summon Carbuncle") && Core.Player.Pet != null, new Action(r => ActionManager.StopCasting())),
                         Spell.Cast(r => "Summon Carbuncle", r => !hasPet && ActionManager.HasSpell("Summon Carbuncle"), r => Core.Player),


                         Spell.Cast(r => WhichPet.Value, r => !string.IsNullOrEmpty(WhichPet)),
                         Spell.Cast("Ruin IV"),

                         Spell.Cast("Enkindle Bahamut"),
                         Spell.Cast("Gemshine"),

                         Spell.Cast("Dreadwyrm Trance"),
                         Spell.Cast("Astral Flow"),


                         Spell.Cast("Aethercharge", r => SummonerARM.AvailablePets == AvailablePetFlags.None),
                         Spell.Cast("Energy Drain"),
                         Spell.Cast("Fester"),


                         Spell.Cast("Ruin")

                        )));
        }
    }
}